import http.client
import json
from django.http import HttpResponse
from .Config import PasswordInit
from .logging_file import logger


class GrantUserInfoBase(PasswordInit):


    def get_user_info(self, request):
        raise NotImplementedError("Subclasses must implement this method to handle specific grant types.")

class PasswordGrantUserInfo(GrantUserInfoBase):
    def get_user_info(self, request):

        access_token = self.set_get_access.get_access()

        logger.info("Access token for user info:")
        logger.info(access_token)

        conn = http.client.HTTPSConnection(self.base_url)
        headers = {'Authorization': f'Bearer {access_token}'}

        try:
            conn.request('GET', '/moas/rest/oauth/getuserinfo', headers=headers)
            response = conn.getresponse()
            response_body = response.read().decode()

            if response.status == 200:
                user_data = json.loads(response_body)
                logger.info("User info retrieved:")
                logger.info(user_data)
                self.set_get_token.set_decoded_token(user_data)
                return HttpResponse(f"User Data: {user_data}")
            else:
                return HttpResponse(f"Failed with status code {response.status}", status=response.status)
        except Exception as e:
            return HttpResponse(f"Error: {str(e)}", status=500)
        finally:
            conn.close()



import http.client
import urllib.parse
import json
import logging
from django.http import HttpResponse
from .Config import PasswordInit
from .logging_file import logger


class GrantTokenBase(PasswordInit):

    def get_access_token(self, request):
        raise NotImplementedError("Subclasses must implement this method to handle specific grant types.")

class PasswordGrantToken(GrantTokenBase):

    def get_access_token(self, request):
        conn = http.client.HTTPSConnection(self.base_url)
        payload = urllib.parse.urlencode({
            'grant_type': 'password',
            'client_id': self.client_id,
            'client_secret': self.client_secret,
            'username': self.username,
            'password': self.password
        })
        headers = {'Content-Type': 'application/x-www-form-urlencoded'}

        try:
            conn.request('POST', '/moas/rest/oauth/token', body=payload, headers=headers)
            response = conn.getresponse()

            if response.status == 200:
                response_body = response.read().decode()
                result = json.loads(response_body)
                access_token = result.get('access_token')
                logger.info("Access token in PasswordGrantToken:")
                logger.info(access_token)
                self.set_get_access.set_access(access_token)
                return HttpResponse(f"Access token: {access_token}")
            else:
                return HttpResponse(f"Failed with status code {response.status}", status=response.status)
        except Exception as e:
            return HttpResponse(f"Error: {str(e)}", status=500)
        finally:
            conn.close()



import http.client
import json
import urllib.parse
#import jwt
from django.http import HttpResponse
from .Config import GrantInit
from .logging_file import logger
import urllib.parse
from .oauth_utils import decode_and_respond_token


class GrantCallbackBase(GrantInit):

    def grant_callback(self, request):

        logger.info("in GrantCallbackBase grant_callback")
        state = request.GET.get('state')
        logger.info("state")

        stored_state = self.set_get_state.get_state()

        if state != stored_state:
            return HttpResponse("State mismatch", status=400)

        code = request.GET.get('code')
        logger.info(code)
        logger.info("code")
        logger.info(code)

        if code:
            logger.info("in if code:")
            payload = self.create_payload(code)
            token = self.request_token(payload)
        else:
            logger.info("in else:")
            token = request.GET.get('id_token')

        logger.info("outside code")
        if token:
            logger.info("in token")
            logger.info(token)
            return decode_and_respond_token(token, self.certificate, self.client_id)
        else:
            return HttpResponse("----No token received----")

    def create_payload(self, code):
        logger.info("in create_payload")
        raise NotImplementedError("Subclasses must implement this method")

    @staticmethod
    def request_token(payload):
        logger.info("in request_token")
        conn = http.client.HTTPSConnection('v.xecurify.com')
        logger.info("conn")
        headers = {'Content-Type': 'application/x-www-form-urlencoded'}
        try:
            logger.info("in try block")
            conn.request('POST', '/moas/rest/oauth/token', body=payload, headers=headers)
            response = conn.getresponse()
            logger.info(response)
            if response.status == 200:
                logger.info("in if ")
                response_body = response.read().decode()
                logger.info("print result")
                result = json.loads(response_body)
                logger.info(result)
                return result.get('id_token')
            else:
                logger.info("in else")
                return None
        finally:
            conn.close()


class AuthorizationCodeGrantCallback(GrantCallbackBase):
    def create_payload(self, code):
        return urllib.parse.urlencode({
            'grant_type': 'authorization_code',
            'client_id': self.client_id,
            'client_secret': self.client_secret,
            'redirect_uri': self.redirect_uri,
            'code': code
        })


class PKCEGrantCallback(GrantCallbackBase):
    def create_payload(self, code):

        code_verifier = self.set_get_verifier.get_verifier()

        return urllib.parse.urlencode({
            'grant_type': 'authorization_code',
            'client_id': self.client_id,
            'client_secret': self.client_secret,
            'redirect_uri': self.redirect_uri,
            'code': code,
            'code_verifier': code_verifier
        })


class ImplicitGrantCallback(GrantCallbackBase):

    def create_payload(self, code):
        logger.info("in ImplicitGrantCallback create_payload")
        pass



import urllib.parse
from django.shortcuts import redirect
from .Config import GrantInit
from .logging_file import logger
from .oauth_utils import generate_random_state, generate_code_verifier, generate_code_challenge

class GrantRedirectBase(GrantInit):

    def grant_redirect(self, request):


        state = generate_random_state()
        logger.info("grant_redirect")

        payload = {
            'client_id': self.client_id,
            'redirect_uri': self.redirect_uri,
            'scope': 'openid',
            'response_type': 'code',
            'state': state,
        }

        logger.info("set payload")
        self.modify_payload_for_grant_type(payload)


        query_string = urllib.parse.urlencode(payload)
        logger.info("query_string")
        url = f"{self.base_url}?{query_string}"


        self.set_get_state.set_state(state)

        return redirect(url)

    def modify_payload_for_grant_type(self, payload):
        logger.info("in modify_payload_for_grant_type")
        raise NotImplementedError("Subclasses must implement this method")

class AuthorizationCodeGrantRedirect(GrantRedirectBase):
    def modify_payload_for_grant_type(self, payload):
        logger.info("in AuthorizationCodeGrantRedirect modify_payload_for_grant_type")
        pass

class PKCEGrantRedirect(GrantRedirectBase):
    def modify_payload_for_grant_type(self, payload):

        code_verifier = generate_code_verifier()
        code_challenge = generate_code_challenge(code_verifier)
        self.set_get_verifier.set_verifier(code_verifier)

        payload.update({
            'code_challenge': code_challenge,
            'code_challenge_method': 'S256',
        })

        self.set_get_verifier.set_verifier(code_verifier)
        logger.info("in PKCEGrantRedirect modify_payload_for_grant_type")

class ImplicitGrantRedirect(GrantRedirectBase):
    def modify_payload_for_grant_type(self, payload):

        payload['response_type'] = 'token'
        logger.info("in ImplicitGrantRedirect modify_payload_for_grant_type")



import base64
import hashlib
import jwt
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import serialization
import secrets
from django.http import HttpResponse
from .logging_file import logger
import os


def generate_code_verifier():
    code_verifier = base64.urlsafe_b64encode(os.urandom(32)).rstrip(b'=').decode('ascii')
    logger.info("In generate_code_verifier " + code_verifier)
    return code_verifier



def generate_code_challenge(verifier):
    hashed = hashlib.sha256(verifier.encode('ascii')).digest()
    logger.info("In generate_code_challenge hashing code_verifier ")
    code_challenge = base64.urlsafe_b64encode(hashed).rstrip(b'=').decode('ascii')
    logger.info("In generate_code_challenge code challenge generated")
    logger.info(code_challenge)
    return code_challenge


def decode_jwt_token(token, certificate, client_id):

    logger.info("certificate")
    logger.info(certificate)
    logger.info("token")
    logger.info(token)
    logger.info("client_id")
    logger.info(client_id)

    public_key = serialization.load_pem_public_key(certificate.encode(), backend=default_backend())

    try:
        decoded_token = jwt.decode(token, public_key, algorithms=["RS256"], audience=client_id, leeway=60)
        logger.info("got decoded_token")
        logger.info(decoded_token)
        return decoded_token
    except jwt.ExpiredSignatureError:
        logger.info("Token expired")
        raise
    except jwt.InvalidTokenError:
        logger.info("Invalid token")
        raise


def decode_and_respond_token(token, certificate, client_id):
    try:
        decoded_token = decode_jwt_token(token, certificate, client_id)
        logger.info("in try of decode_and_respond_token")
        return decoded_token


    except jwt.ExpiredSignatureError:
        logger.error("Token expired")
        return HttpResponse("Token expired")
    except jwt.InvalidTokenError:
        logger.error("Invalid token")
        return HttpResponse("Invalid token")


def generate_random_state(length=16):
    state = secrets.token_urlsafe(length)
    logger.info(f"Generated random state: {state}")
    return state


class Printtoken:

    def __init__(self, request):
        self.request = request

    def set_decoded_token(self, decoded_token):
        logger.info("In set_decoded_token")
        logger.info(decoded_token)
        self.request.session['token'] = decoded_token

    def get_decoded_token(self):
        logger.info("In get_decoded_token")
        return self.request.session.get('token')


class Printverifier:

    def __init__(self, request):
        self.request = request

    def set_verifier(self, decoded_token):
        logger.info("In set_verifier")
        logger.info(decoded_token)
        self.request.session['token'] = decoded_token

    def get_verifier(self):
        logger.info("In get_verifier")
        return self.request.session.get('token')


class Printstate:

    def __init__(self, request):
        self.request = request

    def set_state(self, state):
        logger.info("In set_state")
        logger.info(state)
        # request.session['state'] = state
        self.request.session['state'] = state

    def get_state(self):
        logger.info("In get_state")
        return self.request.session.get('state')


class Printaccess:

    def __init__(self, request):
        self.request = request

    def set_access(self, access_token):
        logger.info("In set_access")
        logger.info(access_token)
        self.request.session['access_token'] = access_token

    def get_access(self):
        logger.info("In get_access")
        logger.info(self.request.session.get('access_token'))
        return self.request.session.get('access_token')





import logging
from logging.handlers import RotatingFileHandler
from datetime import datetime
import os

logger = logging.getLogger('logit')


def setup_logging(log_level, enable_logging):

    if not enable_logging:
        logger.disabled = True
        return

    now = datetime.now().strftime('%Y-%m-%d')
    log_filename = os.path.join(os.getcwd(), f'app_log_{now}.log')

    handler = RotatingFileHandler(log_filename)
    handler.setLevel(log_level)

    formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
    handler.setFormatter(formatter)

    logger.setLevel(log_level)
    logger.addHandler(handler)

    logger.propagate = False





from .oauth_utils import Printtoken, Printverifier, Printstate, Printaccess

class GrantInit:

    def __init__(self, client_id, client_secret, base_url, redirect_uri, certificate, request):

        self.client_id = client_id
        self.client_secret = client_secret
        self.base_url = base_url
        self.redirect_uri = redirect_uri
        self.certificate = certificate
        self.set_get_token = Printtoken(request)
        self.set_get_verifier = Printverifier(request)
        self.set_get_state = Printstate(request)


class PasswordInit:

    def __init__(self, client_id, client_secret, username, password, request):

        self.client_id = client_id
        self.client_secret = client_secret
        self.base_url = 'v.xecurify.com'
        self.username = username
        self.password = password
        self.set_get_token = Printtoken(request)
        self.set_get_access = Printaccess(request)